#!/bin/bash
#

# build_h5py.sh 

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e1[0457]|c2>:<p2711|p2713d|p2714|p2714b>[:<mpich|mvapich2>] <prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
  [[ -n "${basequal}" ]] || ssi_die "Qualifiers mandatory"

  qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
  for qual in "${qlist[@]}"; do
    case $qual in
      e1[0457]|c2)
        cqual=$qual
        ;;
      p2711|p2713d|p2714|p2714b)
        pqual=$qual
        ;;
      mpich|mvapich2)
        bqual=$qual
        export CC="${MPICC:-mpicc}"
        ;;
      *)
        ssi_die "Unrecognized qualifier $qual"
    esac
  done
  if [[ -z "$cqual" ]]; then
    ssi_die "Base qualifier is mandatory"
  fi
  if [[ -z "$pqual" ]]; then
    ssi_die "A python qualifier is mandatory."
  fi
  case ${extraqual} in
    prof)
      :
      ;;
    debug|opt)
      ssi_die "${extraqual} builds of h5py are not supported at this time."
      ;;
    *)
      ssi_die "Unrecognized build qualifier ${extraqual}"
  esac
  basequal=${cqual}${pqual:+:${pqual}}${bqual:+:${bqual}}
}

set_compiler_flags()
{
  case ${cqual} in
    e1[0457])
      cc=gcc
      cxx=false
      fc=false
      ;;
    c2)
      cc=clang
      cxx=false
      fc=false
      ;;
    *)
      ssi_die "Unable to set compiler / options for qualifier ${cqual}."
  esac
    # We don't compile differently if debug, just use different dependencies.
    cflags="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=h5py
origpkgver=v2_7_1
pkgver=${origpkgver}f
ssibuildshims_version=v1_04_04
setuptools_version=v22_0_5 # Required only for python < p2713d.
cython_version=v0_27_3
nose_version=v1_3_7e
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=${package}-${pkgdotver}

get_this_dir
get_ssibuildshims

set --
source ssi_functions
set_qualifiers

source define_basics

set_compiler_flags

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${blddir} || ssi_die "failed to create ${blddir}"

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  ssi_die "Fake declare of $package failed."

setup -B ${package} ${pkgver} -q +${fullqual//:/:+} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "fake setup failed"

(( pv = ${pqual//[A-Za-z]/} ))
if (( ${pv} < 2713 )); then # Bundled with newer python.
  setup -B setuptools ${setuptools_version} -q ${pqual} -z ${product_dir}:${PRODUCTS} || \
    ssi_die "unable to set up UPS package for python setuptools ${setuptools_version}"
fi

ups active

if [ ${OS1} = "Darwin" ]
then
   PATH=`dropit /usr/bin/get-cert $PATH`
fi

set -x

python_shortdotversion=${PYTHON_VERSION#v}
python_shortdotversion=${python_shortdotversion//_/.}
python_shortdotversion=${python_shortdotversion%.*}

# work in a build directory, which will not be part of any distribution
mkdir -p ${pkgdir}/lib/python${python_shortdotversion}/site-packages || ssi_die "failed to create ${pkgdir}/lib/python2.7/site-packages"
cd ${blddir} || ssi_die "Unable to cd into ${blddir}"
tar xf ${tardir}/${pkgtarname}.tar.gz || ssi_die "Failed to untar ${pkgtarname}.tar.gz"
cd ${package}-${pkgdotver} || ssi_die "Failed to cd into ${package}-${pkgdotver}"

hdf5_dotversion=$(echo  "${HDF5_VERSION}" | \
  sed -e 's&^v\([0-9]\{1,\}\)_\([0-9]\{1,\}\)_\([0-9]\{1,\}\)\(_\{0,1\}[a-z].*\)\{0,1\}$&\1.\2.\3&')

setup -B cython $cython_version -q +${pqual}:+prof -z ${fakedb}:${product_dir}:${PRODUCTS} || \
  ssi_die "unable to set up python cython ${cython_version} -q +${pqual} for build."

# Need HDF5_VERSION to be dotversion for python configure to work.
FC=false CXX=false CFLAGS=${cflags} HDF5_VERSION=${hdf5_dotversion} \
    python setup.py --no-user-cfg configure --hdf5=${HDF5} ${bqual:+--mpi} || ssi_die "python setup.py configure failed"

FC=false CXX=false CFLAGS=${cflags} HDF5_VERSION=${hdf5_dotversion} \
    python setup.py --no-user-cfg install --prefix ${pkgdir} || ssi_die "python setup.py install failed"

setup -B nose $nose_version -q +${pqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || \
  ssi_die "unable to set up python nose ${nose_version} -q +${pqual} for tests."

# run some tests
#
if [[ "${bqual}" != mvapich2 ]] && ver_greater 1.14.0 ${NUMPY_VERSION}; then
  FC=false CXX=false CFLAGS=${cflags} HDF5_VERSION=${hdf5_dotversion} \
    python setup.py --no-user-cfg test || ssi_die "python setup.py test failed" || \
    ssi_die "${package} tests failed."
else
  echo "INFO: tests skipped."
fi

set +x

[[ -d "${pkgdir}/lib" ]] || ssi_die "Failed to create ${pkgdir}/lib: something is very wrong."

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  ssi_die "failed to declare ${package} ${pkgver} -q ${fullqual}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q +${fullqual//:/:+} -z ${product_dir}:${PRODUCTS} || \
  ssi_die "Final setup of built product failed."
echo "${package} is installed at ${H5PY_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
